@@ -20,6 +20,7 @@ import json
 import base64
 import functools
 from . import exceptions, utils, user
+import copy
 
 
 API = utils.get_api()
@@ -308,6 +309,23 @@ def unban_user(room_real_id: int, block_id: int, verify: utils.Verify = None):
     return resp
 
 
+def connect_all_livedanmaku(*livedanmaku_classes):
+    """
+    同时连接多个直播间
+    :param livedanmaku_classes: LiveDanmaku类动态参数
+    :return:
+    """
+    tasks = []
+    for room in livedanmaku_classes:
+        task = room.connect(True)
+        tasks.append(task)
+
+    async def run():
+        for task in tasks:
+            await task
+    asyncio.get_event_loop().run_until_complete(run())
+
+
 class LiveDanmaku(object):
     """
     Websocket实时获取直播弹幕
@@ -322,15 +340,17 @@ class LiveDanmaku(object):
     DATAPACK_TYPE_VERIFY = 7
     DATAPACK_TYPE_VERIFY_SUCCESS_RESPONSE = 8
 
-    def __init__(self, room_display_id: int, debug: bool = False, use_wss: bool = True, verify: utils.Verify = None):
+    def __init__(self, room_display_id: int, debug: bool = False, use_wss: bool = True,
+                 verify: utils.Verify = None):
         self.verify = verify
-        self.room_id = room_display_id
-        self.use_wss = use_wss
+        self.room_real_id = room_display_id
+        self.room_display_id = room_display_id
+        self.__use_wss = use_wss
         # logging
-        self.logger = logging.getLogger("LiveDanmaku")
+        self.logger = logging.getLogger(f"LiveDanmaku_{self.room_display_id}")
         self.logger.setLevel(logging.DEBUG if debug else logging.INFO)
         handler = logging.StreamHandler()
-        handler.setFormatter(logging.Formatter("[%(asctime)s][%(levelname)s] %(message)s"))
+        handler.setFormatter(logging.Formatter("[" + str(room_display_id) + "][%(asctime)s][%(levelname)s] %(message)s"))
         self.logger.addHandler(handler)
 
         self.__event_handlers = {}
@@ -338,7 +358,7 @@ class LiveDanmaku(object):
         self.__has_connected = False
         self.__conf = None
 
-    def connect(self):
+    def connect(self, return_task: bool = False):
         """
         连接直播间
         :return:
@@ -346,14 +366,20 @@ class LiveDanmaku(object):
         assert not self.__has_connected, exceptions.LiveException("已连接直播间，不可重复连接")
         # 获取真实房间号
         self.logger.debug("正在获取真实房间号")
-        self.room_id = get_room_play_info(room_display_id=self.room_id, verify=self.verify)["room_id"]
-        self.logger.debug(f"获取成功，真实房间号：{self.room_id}")
+        self.room_real_id = get_room_play_info(room_display_id=self.room_real_id, verify=self.verify)["room_id"]
+        self.logger.debug(f"获取成功，真实房间号：{self.room_real_id}")
         # 获取直播服务器配置
         self.logger.debug("正在获取聊天服务器配置")
-        self.__conf = get_chat_conf(room_real_id=self.room_id, verify=self.verify)
+        self.__conf = get_chat_conf(room_real_id=self.room_real_id, verify=self.verify)
         self.logger.debug("聊天服务器配置获取成功")
         # 连接直播间
-        asyncio.run(self.__main())
+        loop = asyncio.get_event_loop()
+        if return_task:
+            task = loop.create_task(self.__main())
+            return task
+        else:
+            loop.run_until_complete(self.__main())
+            return None
 
     def disconnect(self):
         """
@@ -362,6 +388,9 @@ class LiveDanmaku(object):
         """
         asyncio.gather(self.__ws.close())
 
+    def has_connected(self):
+        return self.__has_connected
+
     async def __main(self):
         """
         入口
@@ -369,8 +398,8 @@ class LiveDanmaku(object):
         """
         self.logger.debug("准备连接直播间")
         for host in self.__conf["host_server_list"]:
-            port = host['wss_port'] if self.use_wss else host['ws_port']
-            protocol = "wss" if self.use_wss else "ws"
+            port = host['wss_port'] if self.__use_wss else host['ws_port']
+            protocol = "wss" if self.__use_wss else "ws"
             uri = f"{protocol}://{host['host']}:{port}/sub"
             self.logger.debug(f"正在尝试连接主机： {uri}")
             try:
@@ -385,7 +414,7 @@ class LiveDanmaku(object):
                             self_info = user.get_self_info(self.verify)
                             uid = self_info["mid"]
                             self.logger.debug(f"用户UID为{uid}")
-                    verifyData = {"uid": 0 if uid is None else uid, "roomid": self.room_id,
+                    verifyData = {"uid": 0 if uid is None else uid, "roomid": self.room_real_id,
                                   "protover": 2, "platform": "web",
                                   "clientver": "1.17.0", "type": 2, "key": self.__conf["token"]}
                     data = json.dumps(verifyData).encode()
@@ -412,6 +441,10 @@ class LiveDanmaku(object):
                 break
             self.logger.debug(f"收到信息：{data}")
             for info in data:
+                callback_info = {
+                    'room_display_id': self.room_display_id,
+                    'room_real_id': self.room_real_id
+                }
                 # 依次处理并调用用户指定函数
                 if info["datapack_type"] == LiveDanmaku.DATAPACK_TYPE_VERIFY_SUCCESS_RESPONSE:
                     # 认证反馈
@@ -424,14 +457,18 @@ class LiveDanmaku(object):
                 elif info["datapack_type"] == LiveDanmaku.DATAPACK_TYPE_HEARTBEAT_RESPONSE:
                     # 心跳包反馈，返回直播间人气
                     self.logger.debug("收到心跳包反馈")
-                    handlers = self.__event_handlers.get("VIEW", [])
+                    callback_info["type"] = 'VIEW'
+                    callback_info["data"] = info["data"]["view"]
+                    handlers = self.__event_handlers.get("VIEW", []) + self.__event_handlers.get("ALL", [])
                     for handler in handlers:
-                        asyncio.create_task(self.__run_in_executor(handler, info["data"]["view"]))
+                        asyncio.create_task(self.__run_as_asynchronous_func(handler, callback_info))
                 elif info["datapack_type"] == LiveDanmaku.DATAPACK_TYPE_NOTICE:
                     # 直播间弹幕、礼物等信息
-                    handlers = self.__event_handlers.get(info["data"]["cmd"], [])
+                    callback_info["type"] = info["data"]["cmd"]
+                    callback_info["data"] = info["data"]
+                    handlers = self.__event_handlers.get(info["data"]["cmd"], []) + self.__event_handlers.get("ALL", [])
                     for handler in handlers:
-                        asyncio.create_task(self.__run_in_executor(handler, info["data"]))
+                        asyncio.create_task(self.__run_as_asynchronous_func(handler, callback_info))
                 else:
                     self.logger.warning("检测到未知的数据包类型，无法处理")
 
@@ -524,27 +561,42 @@ class LiveDanmaku(object):
         return ret
 
     @staticmethod
-    async def __run_in_executor(func, *args):
+    async def __run_as_asynchronous_func(func, *args):
         """
-        将同步程序打包成执行器异步执行
+        区分同步和异步执行函数
         :param func: 回调函数，非异步函数
         :param *args: 传递给函数的参数
         :return:
         """
-        loop = asyncio.get_event_loop()
-        loop.run_in_executor(None, func, *args)
+        if asyncio.iscoroutinefunction(func):
+            await func(*args)
+        else:
+            func(*args)
+
+    def add_event_handler(self, event_name: str, func):
+        """
+        为当前直播弹幕增加事件回调
+        :param event_name: 事件名
+        :param func: 回调函数
+        :return:
+        """
+        if not callable(func):
+            raise exceptions.LiveException("回调函数请传入方法")
+        upper_name = event_name.upper()
+        if upper_name not in self.__event_handlers:
+            self.__event_handlers[upper_name] = []
+        self.__event_handlers[upper_name].append(func)
 
     def on(self, name: str):
         """
         使用@语法，触发事件时会被调用
-        :param name: 事件名，特殊：VIEW（人气更新）
+        :param name: 事件名
         :return:
         """
         """
         直播区更新速度快，以实际API为准，可以开debug自己看
         常见事件名：
         DANMU_MSG: 用户发送弹幕
-        VIEW: 直播间人气更新
         SEND_GIFT: 礼物
         WELCOME: 老爷进入房间
         WELCOME_GUARD: 房管进入房间
@@ -556,6 +608,9 @@ class LiveDanmaku(object):
         ROOM_RANK: 房间排名更新
         INTERACT_WORD: 用户进入直播间
         ACTIVITY_BANNER_UPDATE_V2: 好像是房间名旁边那个xx小时榜
+        本模块自定义事件：
+        VIEW: 直播间人气更新
+        ALL: 所有事件
         """
         def decoration(func):
             upper_name = name.upper()
